@@ -1,18 +1,25 @@
-import ConfigParser
 import logging
 import re
-from subprocess import call, check_call
+
+import six
+from six.moves import configparser
 
 from ironicclient import client, exceptions
+from keystoneclient.v2_0 import client as keystone
+
+from ironic_discoverd import firewall
 
 
 LOG = logging.getLogger("discoverd")
 ALLOW_SEARCH_BY_MAC = True
-CONF = ConfigParser.ConfigParser(
+CONF = configparser.ConfigParser(
     defaults={'debug': 'false',
               'listen_address': '0.0.0.0',
               'listen_port': '5050',
-              'dnsmasq_interface': 'br-ctlplane'})
+              'dnsmasq_interface': 'br-ctlplane',
+              'authenticate': 'true',
+              'firewall_update_period': '15',
+              'ssh_driver_regex': '^.*_ssh$'})
 OS_ARGS = ('os_password', 'os_username', 'os_auth_url', 'os_tenant_name')
 
 
@@ -21,9 +28,14 @@ def get_client():
     return client.get_client(1, **args)
 
 
+def get_keystone(token):
+    return keystone.Client(token=token, auth_url=CONF.get('discoverd',
+                                                          'os_auth_url'))
+
+
 def is_valid_mac(address):
     m = "[0-9a-f]{2}(:[0-9a-f]{2}){5}$"
-    return (isinstance(address, (str, unicode))
+    return (isinstance(address, six.string_types)
             and re.match(m, address.lower()))
 
 
@@ -125,79 +137,14 @@ def process(node_info):
     LOG.info('Node %s was updated with data from discovery process, forcing '
              'power off', node.uuid)
 
-    Firewall.unwhitelist_macs(valid_macs)
-    Firewall.update_filters(ironic)
+    firewall.unwhitelist_macs(valid_macs)
+    firewall.update_filters(ironic)
 
-    ironic.node.set_power_state(node.uuid, 'off')
-
-
-class Firewall(object):
-    MACS_DISCOVERY = set()
-    NEW_CHAIN = 'discovery_temp'
-    CHAIN = 'discovery'
-    INTERFACE = None
-
-    @staticmethod
-    def _iptables(*args, **kwargs):
-        cmd = ('iptables',) + args
-        LOG.debug('Running iptables %s', args)
-        if kwargs.pop('ignore', False):
-            if call(cmd, **kwargs):
-                LOG.warn('iptables failed: %s', args)
-                return False
-            else:
-                return True
-        else:
-            try:
-                return check_call(cmd, **kwargs)
-            except Exception:
-                LOG.error('iptables failed: %s', args)
-                raise
-
-    @classmethod
-    def init(cls):
-        cls.INTERFACE = CONF.get('discoverd', 'dnsmasq_interface')
-        cls._iptables('-F', cls.NEW_CHAIN, ignore=True)
-        cls._iptables('-X', cls.NEW_CHAIN, ignore=True)
-        cls._iptables('-D', 'INPUT', '-i', cls.INTERFACE, '-p', 'udp',
-                      '--dport', '67', '-j', cls.CHAIN,
-                      ignore=True)  # may be missing on first run
-        cls._iptables('-F', cls.CHAIN, ignore=True)
-        cls._iptables('-X', cls.CHAIN, ignore=True)
-        # Code expects it to exist
-        cls._iptables('-N', cls.CHAIN)
-
-    @classmethod
-    def whitelist_macs(cls, macs):
-        cls.MACS_DISCOVERY.update(macs)
-
-    @classmethod
-    def unwhitelist_macs(cls, macs):
-        cls.MACS_DISCOVERY.difference_update(macs)
-
-    @classmethod
-    def update_filters(cls, ironic):
-        macs_active = set(p.address for p in ironic.port.list(limit=0))
-        to_blacklist = macs_active - cls.MACS_DISCOVERY
-
-        # Operate on temporary chain
-        cls._iptables('-N', cls.NEW_CHAIN)
-        # - Blacklist active macs, so that nova can boot them
-        for mac in to_blacklist:
-            cls._iptables('-A', cls.NEW_CHAIN, '-m', 'mac',
-                          '--mac-source', mac, '-j', 'DROP')
-        # - Whitelist everything else
-        cls._iptables('-A', cls.NEW_CHAIN, '-j', 'ACCEPT')
-
-        # Swap chains
-        cls._iptables('-I', 'INPUT', '-i', cls.INTERFACE, '-p', 'udp',
-                      '--dport', '67', '-j', cls.NEW_CHAIN)
-        cls._iptables('-D', 'INPUT', '-i', cls.INTERFACE, '-p', 'udp',
-                      '--dport', '67', '-j', cls.CHAIN,
-                      ignore=True)  # may be missing on first run
-        cls._iptables('-F', cls.CHAIN)
-        cls._iptables('-X', cls.CHAIN)
-        cls._iptables('-E', cls.NEW_CHAIN, cls.CHAIN)
+    try:
+        ironic.node.set_power_state(node.uuid, 'off')
+    except Exception as exc:
+        LOG.error('Failed to power on node %s, check it\'s power '
+                  'management configuration:\n%s', node.uuid, exc)
 
 
 def discover(uuids):
@@ -217,11 +164,16 @@ def discover(uuids):
 
         nodes.append(node)
 
+    if not nodes:
+        LOG.error('No nodes to discover')
+        return
+
     LOG.info('Proceeding with discovery on nodes %s', [n.uuid for n in nodes])
+    ssh_regex = re.compile(CONF.get('discoverd', 'ssh_driver_regex'))
 
     to_exclude = set()
     for node in nodes:
-        if not node.driver.endswith('ssh'):
+        if not ssh_regex.match(node.driver):
             continue
 
         LOG.warn('Driver for %s is %s, requires white-listing MAC',
@@ -232,8 +184,12 @@ def discover(uuids):
         to_exclude.update(p.address for p in ports)
 
     if to_exclude:
-        Firewall.whitelist_macs(to_exclude)
-        Firewall.update_filters(ironic)
+        firewall.whitelist_macs(to_exclude)
+        firewall.update_filters(ironic)
 
     for node in nodes:
-        ironic.node.set_power_state(node.uuid, 'on')
+        try:
+            ironic.node.set_power_state(node.uuid, 'on')
+        except Exception as exc:
+            LOG.error('Failed to power on node %s, check it\'s power '
+                      'management configuration:\n%s', node.uuid, exc)
